/*
   DScope Application

   Copyright (C) 2008,2009:
         Daniel Roggen, droggen@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <QKeyEvent>
#include <QMessageBox>
#include <QPainter>
#include <QDebug>
#include "helpwindowimpl.h"





HelpWindowImpl::HelpWindowImpl(QWidget * parent) 
	: QDialog(parent)
{
	
	setupUi(this);
	
	label->setText(
	"<h3 align=\"center\">DScope</h3>"
	"<p>DScope is a digital oscilloscope optimized for fast rendering and flexibility of configuration. "
	"It consists of a main window with a variable number of scopes displaying the incoming signal traces. "
	"Signals are signed integers with 32-bit resolution at maximum. "
	"</p>"  
	"<p>Signal data sources include TCP/IP sources and serial ports (serial, RFCOMM or USB). "
	"The syntax to specify the data source is explained in the <a href=\"#display\">Input device</a> section below. "
	"</p>"
	"<p>The incoming data can be a plain text format (one line per sample, delimited by a space or a ,;: separator) and binary streaming formats. "
	"The syntax to specify the incoming data format is explained in the <a href=\"#display\">Binary format</a> section below. "
	"</p>"
	"<p>By default one scope is used for each signal trace. "
	"However the number of scopes on display and the signals that they render can be freely configured, with e.g. several traces per scopes. "
	"This is explained in the <a href=\"#display\">Display format</a> section below. "
	"</p>"	   
	"<h4><a name=\"display\">Display format</a></h4>"
	"The way the signal traces are displayed are specified by a display format string. "
	"This string indicates how many scopes are on screen and which traces they should show. "
	"The syntax is: <i>&lt;title;traces[;xscale=xs][;yscale=auto|min max][;color=colors]&gt;&lt;......</i>"
	"<ul>"
	"<li>title: title of the scope</li>"
	"<li>traces: space separated list of traces to display (0=first trace, ...)</li>"
	"<li>xscale: horizontal scale. 1=default. Positive value: zoom in. Negative value: zoomout</li>"
	"<li>yscale: vertical scale. automatic (auto) or manual (min max)</li>"
	"<li>colors: space separated list of trace colors in hexadecimal format RRGGBB </li>"
	"</ul>"
	"For example the following display format string <i>&lt;Calibrated;5 6 7;xscale=1;yscale=-3300 3300 ;color=ff0000 ff00 ff&gt;&lt;Raw;2 3 4;xscale=-2;yscale=auto ;color=ff0000 ff00 ff&gt;</i> "
	"displays two scopes. The first shows traces number 5, 6 and 7, in red, green and blue respectively, with manual vertical scale, default horizontal scale, and the title Calibrated."
	"<h4>Binary format</h4>"
	"The format of the data source is either text or binary, and is described by a binary format string. "
	"An empty binary format string means that the incoming data is plain text. Each line represents one sample and data are separated by one of space, tabulation, colon or semicolon. "
	"Binary format consists of a HEADER followed by raw binary data. "
	"To specify the data format and header the following format string is used: <i>HEADER;C0FMT C1FMT C2FMT....</i> "
	"<ul>"
	"<li>HEADER: plain text header indicating start of data</li>"
	"<li>CxFMT: Format of the data channel x. The format is <i>[-]c|s|S|i|I</i> where c,s|S,i|I indicate an 8, 16 or 32-bit channel. s and i means little-endian. S and I means big-endian. The optional - prefix indicates a signed data channel.</li>"
	"</ul>" 
	"<h4>Input device</h4>"
	"The input device can be a TCP/IP data source or a serial line. It is specified as follows:"
	"<ul>"
	"<li>TCP/IP: <i>tcp:host:port</i></li>"
	"<li>Serial: <i>ser:device:baudrate</i></li>"
	"</ul>"
	"Serial device has the format COMx under Windows or /dev/ttyxxxx under Linux."
	"<h4>User interface</h4>"
	"Scopes can be selected, scaled, and enlarged by using the mouse. "
	"<ul>"
	"<li>Left click: select a scope</li>"
	"<li>Right click: enlarges the selected scope</li>"
	"<li>Wheel up/down: vertical zoom.</li>"
	"<li>Middle click: automatic vertical scale</li>"
	"<li>Shift+wheel up/down: horizontal zoom.</li>"
	"<li>Shift+middle click: default horizontal scale</li>"
	"</ul>"
	"<h4>Command line</h4>"
	"Command line parameters are: <i>-c -f file</i>.<br>"
	"-f specifies a configuration file to load on startup. -c automatically connects to the device (typically used with -c)."
	);
	
}


